"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScriptoriumLambda = void 0;
const server_services_core_1 = require("@fluidframework/server-services-core");
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
class ScriptoriumLambda {
    constructor(opCollection, context, tenantId, documentId) {
        this.opCollection = opCollection;
        this.context = context;
        this.tenantId = tenantId;
        this.documentId = documentId;
        this.pending = new Map();
        this.current = new Map();
    }
    handler(message) {
        const boxcar = (0, server_services_core_1.extractBoxcar)(message);
        for (const baseMessage of boxcar.contents) {
            if (baseMessage.type === server_services_core_1.SequencedOperationType) {
                const value = baseMessage;
                // Remove traces and serialize content before writing to mongo.
                value.operation.traces = [];
                const topic = `${value.tenantId}/${value.documentId}`;
                let pendingMessages = this.pending.get(topic);
                if (!pendingMessages) {
                    pendingMessages = [];
                    this.pending.set(topic, pendingMessages);
                }
                pendingMessages.push(value);
            }
        }
        this.pendingOffset = message;
        this.sendPending();
        return undefined;
    }
    close() {
        this.pending.clear();
        this.current.clear();
        return;
    }
    sendPending() {
        // If there is work currently being sent or we have no pending work return early
        if (this.current.size > 0 || this.pending.size === 0) {
            return;
        }
        // Swap current and pending
        const temp = this.current;
        this.current = this.pending;
        this.pending = temp;
        const batchOffset = this.pendingOffset;
        const allProcessed = [];
        // Process all the batches + checkpoint
        for (const [, messages] of this.current) {
            const processP = this.processMongoCore(messages);
            allProcessed.push(processP);
        }
        Promise.all(allProcessed).then(() => {
            this.current.clear();
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            this.context.checkpoint(batchOffset);
            this.sendPending();
        }, (error) => {
            this.context.error(error, { restart: true });
        });
    }
    async processMongoCore(messages) {
        return this.insertOp(messages);
    }
    async insertOp(messages) {
        const dbOps = messages.map((message) => (Object.assign(Object.assign({}, message), { mongoTimestamp: new Date(message.operation.timestamp) })));
        return (0, server_services_core_1.runWithRetry)(async () => this.opCollection.insertMany(dbOps, false), "insertOpScriptorium", 3 /* maxRetries */, 1000 /* retryAfterMs */, (0, server_services_telemetry_1.getLumberBaseProperties)(this.documentId, this.tenantId), (error) => error.code === 11000 /* shouldIgnoreError */);
    }
}
exports.ScriptoriumLambda = ScriptoriumLambda;
//# sourceMappingURL=lambda.js.map